from __future__ import absolute_import
import os

from flask import Flask, g
from osconf import config_from_environment


class AppSetup(object):

    def __init__(self):
        from backend.pool import Pool
        from .api import GisceStg
        self.api = GisceStg(prefix='/api')
        self.pool = Pool()

    def create_app(self, **config):
        """
        Create a gisce_stg app
        :param config:
        :return: gisce_stg app
        """
        app = Flask(__name__, static_folder=None)

        if 'SQLITE_DB' in os.environ:
            app.config['SQLITE_DB'] = os.environ['SQLITE_DB']

        app.config.update(config)

        self.configure_api(app)
        STGStandAlone.configure_stg(app)
        self.configure_backend(app)

        return app

    def configure_api(self, app):
        """
        Configure different API endpoints
        :param app: Flask application
        :return:
        """
        from .api import resources
        for resource in resources:
            self.api.add_resource(*resource)

        self.api.init_app(app)

    @staticmethod
    def configure_counter(app):
        """
        Configure SqliteCounter counter
        :param app:
        :return:
        """
        from .SqliteCounter import SqliteCounter
        app.counter = SqliteCounter(app.config['SQLITE_DB'])
        return app

    def setup_backend_conn(self):
        try:
            client = self.pool.connect(**config_from_environment('PEEK'))
            g.backend_cnx = client
        except Exception:
            pass

    def configure_backend(self, app):
        app.before_request(self.setup_backend_conn)


class STGStandAlone(object):

    @staticmethod
    def create_app(**config):
        app = Flask(__name__, static_folder=None)
        app.config.update(config)

        STGStandAlone.configure_stg(app)
        return app

    @staticmethod
    def configure_stg(app):
        from .stg import Stg, resources

        api = Stg(prefix='/stg')
        for resource in resources:
            api.add_resource(*resource)
        api.init_app(app, add_specs=False)



from marshmallow import Schema, fields


class ConcentratorSchema(Schema):
    id = fields.Int()
    number = fields.Str()
    name = fields.Str(attribute="name")
    ip_address = fields.Str(attribute="ip_address")
    model = fields.Str(attribute="model")
    type = fields.Str(attribute="type")
    dc_address = fields.Str(attribute="dc_ws_address")
    ws_ip = fields.Str(attribute="stg_ws_ip_address")
    ws_pass = fields.Str(attribute="stg_ws_password")
    ws_version = fields.Str(attribute="dc_ws_stg_version")
    admin_pass = fields.Str(attribute="w_password")
    web_address = fields.Str(attribute="dc_web_address")


class RegisterSchema(Schema):
    id = fields.Int()
    number = fields.Str()
    name = fields.Str(attribute="name")
    tg_name = fields.Str(attribute="name")
    cnc = fields.Nested(ConcentratorSchema, attribute="cnc_id")



# -*- encoding: utf-8 -*-
from spyne import Application, ServiceBase, Mandatory, Boolean, UnsignedInt, UnsignedInteger, Int, Integer, Iterable, ServiceBase, String, Unicode, rpc
from spyne.protocol.soap import Soap11
from spyne.server.wsgi import WsgiApplication

from werkzeug.middleware.dispatcher import DispatcherMiddleware

import logging
from datetime import datetime
from os import path
import re
from flask import Flask
from osconf import config_from_environment


logger = logging.getLogger('ws_stg')


DEFAULT_SAVE_PATH = '/tmp'


dc_status = {
        '0': "SUCCESS: The request with ID {} for the concentrator {} have "
             "finished successfully. {}",
        '1': "WORKING: The request with ID {} for the concentrator {} is "
             "still in progress. {}",
        '2': "ERROR: The request with ID {} for the concentrator {} has "
             "been cancelled due to a timeout on the DC-Meter connection. "
             "{}",
        '3': "ERROR: The request with ID {} for the concentrator {} has "
             "been rejected for being outdated. {}",
        '4': "ERROR: The request with ID {} for the concentrator {} has "
             "been partially applied (success in some meters, failure in "
             "others). {}",
        '5': "ERROR: The request with ID {} for the concentrator {} is "
             "not properly formed."
             " Unable to process it. {}",
        '6': "ERROR: The request with ID {} for the concentrator {} has "
             "been cancelled due to a timeout on the DC-STG connection. "
             "{}",
        '7': "ERROR: The request with ID {} for the concentrator {} has "
             "asked information from a meter which does not exist in the"
             "DC database. {}",
        '8': "ERROR: The request with ID {} for the concentrator {} has an "
             "incompatibility with the protocol version. {}"
    }

meter_status = {
        '0': "SUCCESS: The meter {} have finished serving the request "
             "with ID {} successfully.",
        '1': "ERROR: The meter {} couldn't finish serving the request "
             "with ID {}.",
        '2': "WARNING: The meter {} have finished serving the request with "
             "ID {}. But any report after it is missing",
        '3': "ERROR: The meter {} couldn't finish serving the request with "
             "ID {}. Order partially applied (part of the data sent in the "
             "order were not correctly applied in the meter)."
    }


def get_report_path():
    try:
        return config_from_environment('PEEK')['report_path']
    except Exception as e:
        logger.info(
            'Error reading report save path. setting default {}'.format(
                DEFAULT_SAVE_PATH
            )
        )
        return DEFAULT_SAVE_PATH


class WSSTG(object):

    def __init__(self, filepath=DEFAULT_SAVE_PATH):
        self.save_filepath = filepath

    def manage_report(self, id_pet, id_dc, req_status, file_format, payload):
        status_txt = dc_status.get(str(req_status), '').format(id_pet, id_dc, '')
        txt = "{}:{}:{}:{} A Report has been received with status {}:\n {}".format(
            id_pet, id_dc, req_status, file_format, status_txt,
            payload
        )
        m = re.search('IdRpt="([^"]+)"', payload)
        report = m and m.group(1) or 'S00'

        filename = '{}_{}_{}_{}_{}'.format(
            id_dc, id_pet, report, file_format, datetime.now().strftime('%Y%m%d%H%M%S')
        )
        filepath = path.join(self.save_filepath, filename)
        logger.info('Saving {} to {}'.format(report, filepath))
        with open(path.join(self.save_filepath, filename),'w') as fp:
            fp.write(payload)
        logger.info(txt)

    def manage_request_status(self, id_pet, id_dc, req_status, reference=''):
        status_txt = dc_status.get(str(req_status), '').format(id_pet, id_dc, reference)
        txt = "{}:{}:{}:{} Request status received {}".format(
            id_pet, id_dc, req_status, reference, status_txt
        )
        logger.info(txt)

    def manage_meter_status(self, id_pet, id_dc, id_meters, meter_status, err_cat, err_code):
        status_txt = dc_status.get(str(meter_status), '').format(id_pet, id_dc, meter_status)
        txt = "{}:{}:{}:{}:{}:{} Meter status received {}".format(
            id_pet, id_dc, id_meters, meter_status, err_cat, err_code, status_txt
        )
        logger.info(txt)


class WSSTGService(ServiceBase):

    @rpc(Mandatory(UnsignedInt), String, Mandatory(Int), Mandatory(Int), String, _returns=Mandatory(Boolean))
    def Report(ctx, IdPet, IdDC, ReqStatus, Format, Payload):
        wsstg = WSSTG(filepath=get_report_path())
        wsstg.manage_report(IdPet, IdDC, ReqStatus, Format, Payload)
        return True

    @rpc(Mandatory(UnsignedInt), String, Mandatory(Int), String, _returns=Mandatory(Boolean))
    def UpdateRequestStatus(ctx, IdPet, IdDC, ReqStatus, Reference):
        wsstg = WSSTG()
        wsstg.manage_request_status(IdPet, IdDC, ReqStatus, Reference)
        return True

    @rpc(Mandatory(UnsignedInt), String(nillable=False), String(nillable=False), Mandatory(Int(min_occurs=1)), Int(nillable=True, min_occurs=1), Int(min_occurs=1, nillable=True), _returns=Mandatory(Boolean))
    def UpdateMetersStatus(ctx, IdPet, IdDC, IdMeters, MeterStatus, ErrCat, ErrCode):
        wsstg = WSSTG()
        wsstg.manage_meter_status(IdPet, IdDC, IdMeters, MeterStatus, ErrCat, ErrCode)
        return True


def create_app(**config):
    app = Flask(__name__)
    app.config.update(config)


    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger('ws_stg')

    txt = "WS_STG version 0.1. GISCE-TI S.L"
    logger.info(txt)

    report_save_path = get_report_path()
    logger.info(' * Report save path: {}'.format(report_save_path))

    application = Application([WSSTGService],
        tns='http://www.asais.fr/ns/Saturne/STG/ws',
        in_protocol=Soap11(validator='lxml'),
        out_protocol=Soap11()
    )

    app.wsgi_app = DispatcherMiddleware(app.wsgi_app, {
        '/ws': WsgiApplication(application)
    })

    return app


app = create_app()

if __name__ == '__main__':
    app.run()



import socket
import subprocess
from datetime import datetime, timedelta, date
from urlparse import urlparse

from flask import jsonify, Response, abort, request, current_app
from flask_restplus import Resource, Api
from primestg.service import Service, format_timestamp
from primestg.utils import DLMSTemplates, PrimeTemplates
from primestg.ziv_service import ZivService
from werkzeug.exceptions import MethodNotAllowed

from openstg.models import *
from openstg.utils import ArgumentsParser, get_season
from openstg.config import AppSetup

supported_reports = {
    'instant-data': ['sync', 'reg_only'],  # PRIME code: S01
    'daily-incremental': ['async', 'reg_n_cnc'],  # PRIME code: S02
    'monthly-billing': ['async', 'reg_n_cnc'],  # PRIME code: S04
    'daily-absolute': ['async', 'reg_n_cnc'],  # PRIME code: S05
    'meter-parameters': ['async', 'reg_only'],  # PRIME code: S06
    'meter-events': ['async', 'reg_n_cnc'],  # PRIME code: S09
    'cnc-parameters': ['async', 'cnc_only'],  # PRIME code: S12
    'cutoffs-status': ['async', 'reg_n_cnc'], # PRIME code: S18
    'advanced-instant-data': ['sync', 'reg_only'],  # PRIME code: S21
    'contract-definition': ['async', 'reg_n_cnc'],  # PRIME code: S23
    'cnc-meters': ['async', 'cnc_only'],  # PRIME code: S24
}

supported_orders = {
    'cutoff-reconnect': ['async', 'reg_only'],  # PRIME code: B03
    'order-request': ['async', 'cnc_only'],  # PRIME code: B11
}

class GisceStg(Api):
    pass


class BaseResource(Resource):
    pass


def get_url_from_cnc(cnc_name):
    """
    Gets the web services URL from the DB. Using the concentrator name.
    :param cnc_name: Concentrator name
    :return: An string with the URL of the web service
    """
    try:
        search_params, limit, offset = ArgumentsParser.parse()
    except (ValueError, SyntaxError) as e:
        response = jsonify({
            'status': 'ERROR',
            'errors': {'filter': e.message}
        })
        response.status_code = 422
        return response
    try:
        search_params = [('name', '=', cnc_name)]
        model = ConcentratorModel()
        result = model.get(search_params, limit=limit,
                           offset=offset)
    except ModelException:
        response = jsonify({'status': 'ERROR'})
        response.status_code = 422
        return response

    return '{}'.format(result.items[0]['dc_address'])


def get_field_from_cnc(cnc_name, fieldname):
    """
    Gets the web services URL from the DB. Using the concentrator name.
    :param cnc_name: Concentrator name
    :return: An string with the URL of the web service
    """
    try:
        search_params, limit, offset = ArgumentsParser.parse()
    except (ValueError, SyntaxError) as e:
        response = jsonify({
            'status': 'ERROR',
            'errors': {'filter': e.message}
        })
        response.status_code = 422
        return response
    try:
        search_params = [('name', '=', cnc_name)]
        model = ConcentratorModel()
        result = model.get(search_params, limit=limit,
                           offset=offset)
    except ModelException:
        response = jsonify({'status': 'ERROR'})
        response.status_code = 422
        return response
    if fieldname in result.items[0]:
        return '{}'.format(result.items[0][fieldname])
    return None


def get_cnc_url_from_register(register_name):
    """
    Gets the web services URL from the DB. Using the register name.
    :param register_name: The name of the register
    :return: The URL of the web services of the concentrator
    """
    try:
        search_params, limit, offset = ArgumentsParser.parse()
    except (ValueError, SyntaxError) as e:
        response = jsonify({
            'status': 'ERROR',
            'errors': {'filter': e.message}
        })
        response.status_code = 422
        return response
    try:
        register_name = register_name.split(',')[0]
        search_params = [('name', '=', register_name)]
        model = RegisterModel()
        result = model.get(search_params, limit=limit,
                           offset=offset)
    except ModelException:
        response = jsonify({'status': 'ERROR'})
        response.status_code = 422
        return response
    return '{}'.format(result.items[0]['cnc']['dc_address'])


def get_required_info(request):
    """
    Gets the information needed to ask for every report
    :param request: Contains the parameters given in the path
    :return: A dictionary containing the required information
    """
    now = datetime.now()

    date_from = format_timestamp(now)
    date_to = format_timestamp(now + timedelta(hours=1))

    if request.args.get('from'):
        date_from = '{}{}{}'.format(request.args.get('from'), '000',
                                    get_season(request.args.get('from')))
    if request.args.get('to'):
        date_to = '{}{}{}'.format(request.args.get('to'), '000',
                                  get_season(request.args.get('to')))
    AppSetup.configure_counter(current_app)

    return {
        'date_from': date_from,
        'date_to': date_to,
        'source': request.args.get('source'),
        'request_id': current_app.counter.next()
    }


def test_cnc_connection(public_ip, dc_ws_port=None):
    status = False
    if public_ip:
        status = subprocess.call('/bin/ping -c 1 {}'.format(public_ip), shell=True) == 0
        if not status and dc_ws_port:
            s = socket.socket()
            s.settimeout(10.0)
            error_ws_conn = s.connect_ex((public_ip, dc_ws_port))
            s.close()
            status = error_ws_conn == 0
    return status


class ApiCatchall(BaseResource):

    def get(self, path):
        abort(404)

    post = get
    put = get
    delete = get
    patch = get


class ReadOnlyResource(BaseResource):

    def not_allowed(self):
        raise MethodNotAllowed

    post = patch = not_allowed


# Utilities


class SupportedReports(ReadOnlyResource):

    def get(self):
        return jsonify(supported_reports)


class SupportedOrders(ReadOnlyResource):

    def get(self):
        return jsonify(supported_orders)


class StatusCNC(ReadOnlyResource):

    def post(self, cnc_name):
        url = get_url_from_cnc(cnc_name)
        parse = urlparse(url)
        public_ip = parse.hostname
        dc_ws_port = parse.port
        connects = test_cnc_connection(public_ip, dc_ws_port)
        return jsonify({'status': ('up' if connects else 'down')})


class PingMachine(ReadOnlyResource):

    def post(self, public_ip, dc_ws_port):
        resp_ping = test_cnc_connection(public_ip, int(dc_ws_port))
        return jsonify({'status': ('up' if resp_ping else 'down')})


# Send Orders

#PRIME code: B11
class OrderRequest(ReadOnlyResource):

    def get(self, cnc_name, txx):
        info = get_required_info(request)
        generic_values = {
            'id_req': 'B11',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'version': get_field_from_cnc(cnc_name, 'ws_version')
        }
        payload = {
            'txx': txx,
            'date_from': info['date_from'],
            'date_to': info['date_to']
        }
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        order = s.get_order_request(generic_values, payload)
        resp = Response(order, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# DLMS Templates
class DLMStemplates(ReadOnlyResource):
    def get(self):
        dlms_templates = DLMSTemplates().templates
        return jsonify([{'name': key, 'description': dlms_templates[key]['description']} for key in dlms_templates])


# DLMS Templates by Type
class DLMStemplatesByType(ReadOnlyResource):
    def get(self, template_type):
        return DLMSTemplates().get_available_templates(template_type=template_type)


# PRIME code: B12
class ChangeDLMScontracts(ReadOnlyResource):
    def post(self, cnc_name, dlms_template):
        meter_name = request.json.get('meter_name', None)
        activation_date_text = request.json.get('activation_date', None)
        info = get_required_info(request)

        generic_values = {
            'id_req': 'B12',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'cnt': meter_name,
            'version': get_field_from_cnc(cnc_name, 'ws_version'),
        }

        try:
            url = get_url_from_cnc(cnc_name)

            if not url or url == 'None':
                return self.im_a_teapot('No url from Cnc {}'.format(cnc_name))

            s = Service(info['request_id'], url, sync=True)

            if activation_date_text is None:
                activation_date = datetime.now() + timedelta(days=1)
            else:
                activation_date = datetime.strptime(activation_date_text, '%Y-%m-%d')

            payload = {
                'template': dlms_template,
                'date': activation_date,
                'date_to': format_timestamp(datetime.now()+timedelta(hours=1)),
                'date_from': format_timestamp(datetime.now()),
            }

            order = s.order_raw_dlms(generic_values, payload)
            resp = Response(order, mimetype="text/plain")
            resp.headers['request_id'] = info['request_id']
            return resp.response
        except Exception as e:
            return e

    def im_a_teapot(self, message):
        response = jsonify({'message': message})
        response.status_code = 418
        return response

# PRIME code: B12
class ChangeDLMScurrentpowers(ReadOnlyResource):
    def post(self, cnc_name):
        meter_name = request.json.get('meter_name', None)
        p1 = request.json.get('p1', None)
        p2 = request.json.get('p2', None)
        p3 = request.json.get('p3', None)
        p4 = request.json.get('p4', None)
        p5 = request.json.get('p5', None)
        p6 = request.json.get('p6', None)

        info = get_required_info(request)

        generic_values = {
            'id_req': 'B12',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'cnt': meter_name,
            'version': get_field_from_cnc(cnc_name, 'ws_version'),
        }

        try:
            url = get_url_from_cnc(cnc_name)

            if not url or url == 'None':
                return self.im_a_teapot('No url from Cnc {}'.format(cnc_name))

            s = Service(info['request_id'], url, sync=True)

            payload = {
                'template': 'C1_ACT_POWERS',
                'powers': [p1, p2, p3, p4, p5, p6],
                'date_to': format_timestamp(datetime.now()+timedelta(hours=1)),
                'date_from': format_timestamp(datetime.now()),
            }

            order = s.order_raw_dlms(generic_values, payload)
            resp = Response(order, mimetype="text/plain")
            resp.headers['request_id'] = info['request_id']
            return resp.response
        except Exception as e:
            return e

    def im_a_teapot(self, message):
        response = jsonify({'message': message})
        response.status_code = 418
        return response


class ChangeDLMSlatentpowers(ReadOnlyResource):
    def post(self, cnc_name):
        meter_name = request.json.get('meter_name', None)
        p1 = request.json.get('p1', None)
        p2 = request.json.get('p2', None)
        p3 = request.json.get('p3', None)
        p4 = request.json.get('p4', None)
        p5 = request.json.get('p5', None)
        p6 = request.json.get('p6', None)
        textdate = request.json.get('date', None)

        info = get_required_info(request)

        generic_values = {
            'id_req': 'B12',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'cnt': meter_name,
            'version': get_field_from_cnc(cnc_name, 'ws_version'),
        }

        try:
            url = get_url_from_cnc(cnc_name)

            if not url or url == 'None':
                return self.im_a_teapot('No url from Cnc {}'.format(cnc_name))

            s = Service(info['request_id'], url, sync=True)

            payload = {
                'template': 'C1_LAT_POWERS',
                'powers': [p1, p2, p3, p4, p5, p6],
                'date': datetime.strptime(textdate, '%Y-%m-%d').date(),
                'date_to': format_timestamp(datetime.now()+timedelta(hours=1)),
                'date_from': format_timestamp(datetime.now()),
            }

            order = s.order_raw_dlms(generic_values, payload)
            resp = Response(order, mimetype="text/plain")
            resp.headers['request_id'] = info['request_id']
            return resp.response
        except Exception as e:
            return e

    def im_a_teapot(self, message):
        response = jsonify({'message': message})
        response.status_code = 418
        return response


class GetDLMSTrafoRatio(ReadOnlyResource):
    def post(self, cnc_name):
        meter_name = request.json.get('meter_name', None)
        info = get_required_info(request)

        generic_values = {
            'id_req': 'B12',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'cnt': meter_name,
        }

        try:
            url = get_url_from_cnc(cnc_name)

            if not url or url == 'None':
                return self.im_a_teapot('No url from Cnc {}'.format(cnc_name))

            s = Service(info['request_id'], url, sync=True)

            payload = {
                'template': 'TRAFO_RATIO',
                'date_to': format_timestamp(datetime.now() + timedelta(hours=1)),
                'date_from': format_timestamp(datetime.now()),
            }

            order = s.order_raw_dlms(generic_values, payload)
            resp = Response(order, mimetype="text/plain")
            resp.headers['request_id'] = info['request_id']
            return resp.response
        except Exception as e:
            return e

    def im_a_teapot(self, message):
        response = jsonify({'message': message})
        response.status_code = 418
        return response


class UploadZIVCycle(ReadOnlyResource):
    def post(self, cnc_name):
        cycle_data = request.json.get('cycle_data', None)
        info = get_required_info(request)

        try:
            url = get_field_from_cnc(cnc_name, 'web_address')
            username = 'admin'
            password = get_field_from_cnc(cnc_name, 'admin_pass')

            if not url or url == 'None':
                return self.im_a_teapot('No url from Cnc {}'.format(cnc_name))

            zs = ZivService(url, user=username, password=password, sync=True)

            result = zs.send_cycle("cycle", cycle_data)
            return result.status_code
        except Exception as e:
            return e

    def im_a_teapot(self, message):
        response = jsonify({'message': message})
        response.status_code = 418
        return response

# PRIME code: B03
class CutoffReconnection(ReadOnlyResource):

    def post(self, cnc_name):
        info = get_required_info(request)
        meter_name = request.json.get('meter_name', None)
        order = request.json.get('order', None)

        generic_values = {
            'id_req': 'B03',
            'id_pet': str(info['request_id']),
            'cnc': cnc_name,
            'cnt': meter_name,
            'version': get_field_from_cnc(cnc_name, 'ws_version'),
        }

        assert order in ('cutoff', 'reconnect', 'connect'), "Order not recognized"

        if order == 'cutoff':
            payload = {
                'order_param': '0',
            }
        elif order == 'reconnect':
            payload = {
                'order_param': '1'
            }
        elif order == 'connect':
            payload = {
                'order_param': '2'
            }

        payload.update({
            'date_from': info['date_from'],
            'date_to': info['date_to']
        })
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        order = s.get_cutoff_reconnection(generic_values, payload)
        resp = Response(order, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp.response


# Synchronous requests

# PRIME code: S01
class InstantData(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_instant_data(register_name)
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S02
class DailyIncremental(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_daily_incremental(register_name, info['date_from'],
                                      info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllDailyIncremental(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_daily_incremental(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S04
class MonthlyBilling(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_monthly_billing(register_name, info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllMonthlyBilling(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_monthly_billing(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S05
class DailyAbsolute(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_daily_absolute(register_name, info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllDailyAbsolute(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_daily_absolute(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S06
class RegisterParameters(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_meter_parameters(register_name, info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllRegisterParameters(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_meter_parameters(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S09
class RegisterEvents(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_meter_events(register_name, info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllRegisterEvents(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_meter_events(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp

# PRIME code: S12
class ConcentratorParameters(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=True)
        rep = s.get_concentrator_parameters(cnc_name, info['date_from'],
                                            info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S21
class AdvancedInstantData(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_advanced_instant_data(register_name)
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S23
class ContractDefinition(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=True)
        rep = s.get_contract_definition(register_name, info['date_from'],
                                        info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# PRIME code: S24
class ConcentratorMeters(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_concentrator_meters(cnc_name, info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


class AllContractDefinition(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=True)
        rep = s.get_all_contract_definition(info['date_from'], info['date_to'])
        resp = Response(rep, mimetype="text/plain")
        resp.headers['request_id'] = info['request_id']
        return resp


# Asynchronous requests

# PRIME code: S02
class DailyIncrementalAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_daily_incremental(register_name, info['date_from'],
                                      info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllDailyIncrementalAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_daily_incremental(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S04
class MonthlyBillingAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_monthly_billing(register_name, info['date_from'],
                                    info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllMonthlyBillingAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_monthly_billing(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S05
class DailyAbsoluteAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_daily_absolute(register_name, info['date_from'],
                                   info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllDailyAbsoluteAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_daily_absolute(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S06
class RegisterParametersAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_meter_parameters(register_name, info['date_from'],
                                        info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllRegisterParametersAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_meter_parameters(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S09
class RegisterEventsAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_meter_events(register_name, info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllRegisterEventsAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_meter_events(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S12
class ConcentratorParametersAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_concentrator_parameters(cnc_name, info['date_from'],
                                            info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S18
class CutoffsStatusAsync(ReadOnlyResource):
    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_cutoffs_status(register_name, info['date_from'],
                                    info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllCutoffsStatusAsync(ReadOnlyResource):
    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_cutoffs_status(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}

# PRIME code: S23
class ContractDefinitionAsync(ReadOnlyResource):

    def get(self, register_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_cnc_url_from_register(register_name),
                    sync=False, source=info['source'])
        rep = s.get_contract_definition(register_name, info['date_from'],
                                      info['date_to'])
        return rep, {'request_id': info['request_id']}


# PRIME code: S24
class ConcentratorMetersAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name), sync=False, source=info['source'])
        rep = s.get_concentrator_meters(cnc_name, info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


class AllContractDefinitionAsync(ReadOnlyResource):

    def get(self, cnc_name):
        info = get_required_info(request)
        s = Service(info['request_id'], get_url_from_cnc(cnc_name),
                    sync=False, source=info['source'])
        rep = s.get_all_contract_definition(info['date_from'], info['date_to'])
        return rep, {'request_id': info['request_id']}


resources = [
    # Utilities
    (SupportedReports, '/supported_reports'),
    (SupportedOrders, '/supported_orders'),
    (StatusCNC, '/status/<string:cnc_name>'),
    (PingMachine, '/ping/<string:public_ip>/<string:dc_ws_port>'),

    # Synchronous requests
    # PRIME code: S01
    (InstantData, '/instant-data/<string:register_name>'),
    # PRIME code: S02
    (DailyIncremental, '/daily-incremental/<string:register_name>'),
    (AllDailyIncremental, '/<string:cnc_name>/daily-incremental'),
    # PRIME code: S04
    (MonthlyBilling, '/monthly-billing/<string:register_name>'),
    (AllMonthlyBilling, '/<string:cnc_name>/monthly-billing'),
    # PRIME code: S05
    (DailyAbsolute, '/daily-absolute/<string:register_name>'),
    (AllDailyAbsolute, '/<string:cnc_name>/daily-absolute'),
    # PRIME code: S06
    (RegisterParameters, '/meter-parameters/<string:register_name>'),
    (AllRegisterParameters, '/<string:cnc_name>/meter-parameters'),
    # PRIME code: S09
    (RegisterEvents, '/meter-events/<string:register_name>'),
    (AllRegisterEvents, '/<string:cnc_name>/meter-events'),
    # PRIME code: S12
    (ConcentratorParameters, '/<string:cnc_name>/cnc-parameters'),
    # PRIME code: S21
    (AdvancedInstantData, '/advanced-instant-data/<string:register_name>'),
    # PRIME code: S23
    (ContractDefinition, '/contract-definition/<string:register_name>'),
    (AllContractDefinition, '/<string:cnc_name>/contract-definition'),
    # PRIME code: S24
    (ConcentratorMeters, '/<string:cnc_name>/cnc-meters'),


    # Asynchronous requests
    # PRIME code: S02
    (DailyIncrementalAsync, '/daily-incremental/async/<string:register_name>'),
    (AllDailyIncrementalAsync, '/<string:cnc_name>/daily-incremental/async'),
    # PRIME code: S04
    (MonthlyBillingAsync, '/monthly-billing/async/<string:register_name>'),
    (AllMonthlyBillingAsync, '/<string:cnc_name>/monthly-billing/async'),
    # PRIME code: S05
    (DailyAbsoluteAsync, '/daily-absolute/async/<string:register_name>'),
    (AllDailyAbsoluteAsync, '/<string:cnc_name>/daily-absolute/async'),
    # PRIME code: S06
    (RegisterParametersAsync, '/meter-parameters/async/<string:register_name>'),
    (AllRegisterParametersAsync, '/<string:cnc_name>/meter-parameters/async'),
    # PRIME code: S09
    (RegisterEventsAsync, '/meter-events/async/<string:register_name>'),
    (AllRegisterEventsAsync, '/<string:cnc_name>/meter-events/async'),
    # PRIME code: S12
    (ConcentratorParametersAsync, '/<string:cnc_name>/cnc-parameters/async'),
    # PRIME code: S18
    (CutoffsStatusAsync, '/cutoffs-status/async/<string:register_name>'),
    (AllCutoffsStatusAsync, '/<string:cnc_name>/cutoffs-status/async'),
    # PRIME code: S23
    (ContractDefinitionAsync, '/contract-definition/async/<string:register_name>'),
    (AllContractDefinitionAsync, '/<string:cnc_name>/contract-definition/async'),
    # PRIME code: S24
    (ConcentratorMetersAsync, '/<string:cnc_name>/cnc-meters/async'),

    # Send Orders
    # PRIME code: B03
    (CutoffReconnection, '/<string:cnc_name>/cutoff-reconnect/async'),

    # PRIME code: B11
    (OrderRequest, '/<string:cnc_name>/order-request/<string:txx>/async'),

    # DLMS Contracts
    (DLMStemplates, '/dlms_templates'),
    (DLMStemplatesByType, '/dlms_templates/types/<string:template_type>'),
    (ChangeDLMScontracts, '/<string:cnc_name>/dlms_templates/<string:dlms_template>'),
    (ChangeDLMScurrentpowers, '/<string:cnc_name>/currentpowers'),
    (ChangeDLMSlatentpowers, '/<string:cnc_name>/latentpowers'),
    (GetDLMSTrafoRatio, '/<string:cnc_name>/get-traforatio'),

    (UploadZIVCycle, '/<string:cnc_name>/ziv_cycle'),
]



from ast import literal_eval

from flask_restful import reqparse
from datetime import datetime
import pytz

class ArgumentsParser(object):

    @staticmethod
    def parse():
        parser = reqparse.RequestParser()
        parser.add_argument(
            'filter', dest='filter',
            type=str, help='Filter for searching items'
        )
        parser.add_argument(
            'schema', dest='schema',
            type=str, help='Schema for dumping the JSON'
        )
        parser.add_argument(
            'limit', dest='limit', default=10,
            type=int, help='Limit results'
        )
        parser.add_argument(
            'offset', dest='offset', default=0,
            type=int, help='Offset of results'
        )
        args = parser.parse_args()
        limit = args.limit
        offset = args.offset
        search_params = []
        if args.filter:
            try:
                search_params = literal_eval(args.filter)
            except (ValueError, SyntaxError) as e:
                raise
        return search_params, limit, offset


def get_season(dt):
    utc = pytz.timezone('UTC')
    query_date = utc.localize(datetime.strptime(dt, "%Y%m%d%H%M%S"))
    local_tz = pytz.timezone('Europe/Madrid')
    local_time = query_date.astimezone(local_tz)
    if local_time.dst().seconds > 0:
        return 'S'
    else:
        return 'W'



from collections import namedtuple

from backend.utils import unflatdot, normalize, flatdot, make_schema, recursive_crud
from backend.models import get_model

from openstg.serializers import *

SearchResult = namedtuple('SearchResult', ['items', 'n_items', 'limit', 'offset'])


class ModelException(Exception):
    pass


class Model(object):
    schema = unflatdot([])
    serializer = None
    model = None

    def get(self, search_params, limit, offset):
        model = get_model(self.model)
        try:
            n_items = model.search_count(search_params)
            res_ids = model.search(search_params, limit=limit, offset=offset)
        except Exception:
            raise ModelException
        if not res_ids:
            return SearchResult([], 0, limit, offset)
        items = []
        for values in model.read(res_ids, list(self.schema.keys())):
            obj = normalize(model, values, self.schema)
            items.append(self.serializer.dump(obj=obj).data)
        return SearchResult(items, n_items, limit, offset)


class ConcentratorModel(Model):
    model = 'tg.concentrator'
    schema = unflatdot([
        'name',
        'ip_address',
        'model',
        'type',
        'w_password',
        'r_password',
        'fw_version',
        'fw_comm_version',
        'protocol',
        'communication',
        'dc_web_address',
        'dc_ws_address',
        'stg_ws_ip_address',
        'stg_ws_password',
        'modem_user',
        'modem_password',
        'dc_ws_stg_version',
    ])
    serializer = ConcentratorSchema()


class RegisterModel(Model):
    model = 'giscedata.registrador'
    schema = unflatdot([
        'name',
        'cnc_id.id',
        'cnc_id.name',
        'cnc_id.ip_address',
        'cnc_id.dc_ws_address',
        'cnc_id.dc_ws_stg_version',
    ])
    serializer = RegisterSchema()



# coding: utf-8
from flask import Flask, request
from flask_restplus import Api, Resource
import xml.dom.minidom as xmlmd
import time
import os


dc_status = {
        '0': "SUCCESS: The request with ID {} for the concentrator {} have "
             "finished successfully. {}",
        '1': "WORKING: The request with ID {} for the concentrator {} is "
             "still in progress. {}",
        '2': "ERROR: The request with ID {} for the concentrator {} has "
             "been cancelled due to a timeout on the DC-Meter connection. "
             "{}",
        '3': "ERROR: The request with ID {} for the concentrator {} has "
             "been rejected for being outdated. {}",
        '4': "ERROR: The request with ID {} for the concentrator {} has "
             "been partially applied (success in some meters, failure in "
             "others). {}",
        '5': "ERROR: The request with ID {} for the concentrator {} is "
             "not properly formed."
             " Unable to process it. {}",
        '6': "ERROR: The request with ID {} for the concentrator {} has "
             "been cancelled due to a timeout on the DC-STG connection. "
             "{}",
        '7': "ERROR: The request with ID {} for the concentrator {} has "
             "asked information from a meter which does not exist in the"
             "DC database. {}",
        '8': "ERROR: The request with ID {} for the concentrator {} has an "
             "incompatibility with the protocol version. {}"
    }

meter_status = {
        '0': "SUCCESS: The meter {} have finished serving the request "
             "with ID {} successfully.",
        '1': "ERROR: The meter {} couldn't finish serving the request "
             "with ID {}.",
        '2': "WARNING: The meter {} have finished serving the request with "
             "ID {}. But any report after it is missing",
        '3': "ERROR: The meter {} couldn't finish serving the request with "
             "ID {}. Order partially applied (part of the data sent in the "
             "order were not correctly applied in the meter)."
    }


class Stg(Api):
    def _register_doc(self, app_or_blueprint):
        #We have to
        pass


def getText(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc)


class WSStgResource(Resource):

    def post(self):
        f_name = '/tmp/post-{}'.format(time.time())
        if isinstance(request.data, bytes):
            # Python3 backwards compatibility
            request.data = request.data.decode()
        with open(f_name, 'w') as f:
            f.write(request.data)
        out_msg = manage_request()
        os.remove(f_name)
        print(out_msg)
        return "Ok", 200


def manage_request_status(info):
    if 'Reference' not in info:
        info['Reference'] = ''
    default = "ERROR: The request with ID {} for the concentrator {}. has " \
              "suffered an UNKNOWN error {}".format(info['IdPet'],
                                                    info['IdDC'],
                                                    info['Reference'])
    return dc_status.get(info['ReqStatus'], default).format(
        info['IdPet'], info['IdDC'], info['Reference'])


def manage_meter_status(info):
    default = "ERROR: The meter {} suffered and UNKNOWN error processing the " \
              "request with ID {}.".format(info['IdPet'], info['IdDC'])
    return meter_status.get(info['MeterStatus'], default).format(
        info['IdMeters'], info['IdPet'])


def manage_request():
    dom = xmlmd.parseString(request.data.replace('ns1:', ''))
    if 'UpdateMetersStatus' in request.data:
        return manage_meter_status(create_dict(dom, 'UpdateMetersStatus'))
    else:
        if 'UpdateRequestStatus' in request.data:
            return manage_request_status(
                create_dict(dom, 'UpdateRequestStatus'))
        else:  # Report message
            return manager_report_status(create_dict(dom, 'Report'))


def create_dict(dom, item):
    res = {}
    for elem in dom.getElementsByTagName(item):
        for x in elem.childNodes:
            if x.nodeType == xmlmd.Node.ELEMENT_NODE:
                if x.firstChild is None:
                    pass
                else:
                    res[x.tagName] = x.childNodes[0].data
    return res


def manager_report_status(info):
    if 'Reference' not in info:
        info['Reference'] = ''
    default = 'A report has been received but we don\'t handle its type yet'
    return 'A report has been received. With status: {}'.format(
        dc_status.get(info['ReqStatus'], default).format(info['IdPet'],
                                                         info['IdDC'],
                                                         info['Reference']))

    # TODO: Connect to ERP to read the S15 report received


resources = [
    (WSStgResource, '/ws')
]



# -*- coding: utf-8 -*-
import requests
from urlparse import urlparse
import json

class ErpApiException(Exception):
    pass

class ErpApi(object):
    token = None
    base_endpoint = None

    def connect(self, url, token=None):
        parsed_url = urlparse(url)

        self.base_endpoint = '{}://{}:{}{}'.format(
            parsed_url.scheme, parsed_url.hostname, parsed_url.port, parsed_url.path
        )

        if token is None:
            # print('Connect by usr/pwd')
            user = parsed_url.username
            password = parsed_url.password


            token_url = '{}token'.format(self.base_endpoint)

            session = requests.Session()
            session.auth = (user, password)

            r = session.get(token_url)

            if r.status_code == 200:
                data = r.json()
                self.token = data['token']
            else:
                raise(ErpApiException("Error connecting: [{}] {}".format(r.status_code, r.content)))
        else:
            # print('Connect by token')
            self.token = token

    def test(self):
        # test connection with ResCompany get
        res = {}
        try:
            res = self.filter('IrModel', filter=[], fields=['name'], limit=1)
        except Exception as e:
            pass

        return len(res.get('items', [])) and True or False

    def __str__(self):
        test = self.test()
        if self.base_endpoint is not None and self.token is not None:
            if test:
                return 'Connected REST ERP API in {}'.format(self.base_endpoint)
            else:
                return 'Not Connected. BAD CREDENTIALS'
        else:
            return 'Not Connected'

    def create_uri(self, object, res_id=None, filter=None, fields=None, limit=None, offset=None):
        '''
        Creates a URI using parameters
        :param object: Camel case object name (i.e. 'ResUser')
        :param oid: resurce id if desired
        :param filter: ERP domain
        :param fields: list of fields to download in .dot notation (i.e user_id.name)
        :param limit: Number of registers to get (0 -> all)
        :param offset: first register offset
        :return:
        '''
        first_param = True
        uri = '{}{}'.format(self.base_endpoint, object)
        if res_id:
            uri += '/{}'.format(res_id)
        if fields:
            schema = ','.join(fields)
            uri += '{}schema={}'.format(first_param and '?' or '&', schema)
            first_param = False
        if filter:
            if isinstance(filter, (list, tuple)):
                 filter = json.dumps(filter)
            uri += '{}filter={}'.format(first_param and '?' or '&', filter)
            first_param = False
        if limit:
            uri += '{}limit={}'.format(first_param and '?' or '&', limit)
            first_param = False
        if offset:
            uri += '{}offset={}'.format(first_param and '?' or '&', offset)
            first_param = False

        return uri

    def read(self, object, res_id, fields=None):
        '''
        Read model data
        :param object: Camel case object name (i.e. 'ResUser')
        :param id: resurce id
        :param fields: list of fields to download in .dot notation (i.e user_id.name)
        :return: dictionary with field and value
        '''
        headers = {'Authorization': 'token {}'.format(self.token)}

        uri = self.create_uri(object, res_id, fields=fields)
        r = requests.get(uri, headers=headers)

        return r.json()

    def filter(self, object, filter=None, fields=None, limit=None):
        '''
        Search for resurces of a model using a ERP domain (filter)
        :param object: Camel case object name (i.e. 'ResUser')
        :param id: filter ERP style domain
        :param fields: list of fields to download in .dot notation (i.e user_id.name)
        :return: list of dictionary with field and values
        '''
        headers = {'Authorization': 'token {}'.format(self.token)}

        uri = self.create_uri(object, fields=fields, filter=filter, limit=limit)
        r = requests.get(uri, headers=headers)

        return r.json()





import sqlite3
import random
import os
import time


default_filename = 'sqlite_counter.db'

class SqliteCounter():
    def __init__(self, filename):
        if not os.path.isfile(filename):
            self.db_init(filename)
        else:
            self.con = sqlite3.connect(filename)

    def next(self):
        with self.con:
            self.con.execute('UPDATE counter SET value = (value + 1) % 4294967295')
            for row in self.con.execute('SELECT value FROM counter WHERE name="stg"'):
                return row[0]

    def db_init(self, filename):
        self.con = sqlite3.connect(filename)
        with self.con:
            self.con.execute('CREATE TABLE counter (id PRIMARY KEY, value integer, name varchar)')
            self.con.execute('INSERT INTO counter VALUES (1,0,"stg")')

def work():
    counter = SqliteCounter(default_filename)
    pid = os.getpid()
    for num in range(1, 25):
        wait_time = random.randint(0, 2)
        time.sleep(wait_time/100.0)
        res = counter.next()
        print('{}: {}'.format(pid, res))

if __name__ == '__main__':
    work()



from setuptools import setup, find_packages


setup(
    name='openstg',
    version='1.12.1',
    packages=find_packages(),
    url='https://github.com/gisce/openstg',
    license='GNU Affero General Public License v3',
    author='GISCE-TI, S.L.',
    author_email='devel@gisce.net',
    install_requires=[
        'ERPpeek',
        'Flask==0.12.2',
        'Flask-Cors==3.0.2',
        'Flask-Login==0.2.11',
        'Flask-RESTful==0.3.5',
        'flask-restplus==0.10.1',
        'primestg>=1.32.0',
        'marshmallow<3',
        'osconf==0.1.3',
        'pytz',
    ],
    description='API for STG-Protocol-DC Interface Specification'
)



from __future__ import absolute_import
from openstg.config import AppSetup

application = AppSetup().create_app()

if __name__ == '__main__':
    application.run(host='0.0.0.0', debug=True, use_reloader=False)
